from sqlalchemy import and_

from database import db, ITEMS_PER_PAGE
from flask_covid19.blueprints.app_all.all_model import BlueprintDateReported, BlueprintLocationGroup


class RkiMeldedatum(BlueprintDateReported):
    __tablename__ = 'rki_cases_date_reported'
    __mapper_args__ = {'concrete': True}
    __table_args__ = (
        db.UniqueConstraint(
            'date_reported_import_str',
            'datum',
            'year_day_of_year',
            name="uix_rki_cases_date_reported"
        ),
    )

    id = db.Column(db.Integer, primary_key=True)
    datum = db.Column(db.Date, nullable=False, unique=True)
    year = db.Column(db.Integer, nullable=False)
    month = db.Column(db.Integer, nullable=False)
    day_of_month = db.Column(db.Integer, nullable=False)
    day_of_week = db.Column(db.Integer, nullable=False)
    week_of_year = db.Column(db.Integer, nullable=False)
    day_of_year = db.Column(db.Integer, nullable=True)
    year_week = db.Column(db.String(255), nullable=False)
    year_day_of_year = db.Column(db.String(255), nullable=False, unique=True)
    date_reported_import_str = db.Column(db.String(255), nullable=False, unique=True)
    processed = db.Column(db.Boolean, nullable=False)

    @classmethod
    def create_new_object_factory(cls, my_date_rep):
        return super().create_new_object_factory_for_rki(my_date_rep)


class RkiBundesland(BlueprintLocationGroup):
    __tablename__ = 'rki_cases_landkreis_bundesland'
    __mapper_args__ = {'concrete': True}
    __table_args__ = (
        db.UniqueConstraint('id_bundesland', 'region', name="uix_rki_cases_landkreis_bundesland"),
    )

    id = db.Column(db.Integer, primary_key=True)
    id_bundesland = db.Column(db.String(255), nullable=False)
    region = db.Column(db.String(255), nullable=False)


class RkiLandkreis(db.Model):
    __tablename__ = 'rki_cases_landkreis'
    __table_args__ = (
        db.UniqueConstraint('landkreis', 'id_landkreis', name="uix_rki_cases_landkreis"),
    )

    id = db.Column(db.Integer, primary_key=True)
    id_landkreis = db.Column(db.String(255), nullable=False)
    landkreis = db.Column(db.String(255), nullable=False)
    bundesland_id = db.Column(db.Integer, db.ForeignKey('rki_cases_landkreis_bundesland.id'), nullable=False)
    bundesland = db.relationship(
        'RkiBundesland',
        lazy='joined',
        cascade='save-update',
        order_by='RkiBundesland.region')

    def __str__(self):
        result = ""
        result += self.landkreis
        result += " "
        result += self.bundesland.region
        result += " "
        return result

    @classmethod
    def remove_all(cls):
        num_rows_deleted = 0
        try:
            num_rows_deleted = db.session.query(cls).delete()
            db.session.commit()
        except Exception:
            db.session.rollback()
        return num_rows_deleted

    @classmethod
    def get_all_as_page(cls, page):
        return db.session.query(cls)\
            .order_by(cls.country)\
            .paginate(page, per_page=ITEMS_PER_PAGE)

    @classmethod
    def get_all(cls):
        return db.session.query(cls)\
            .order_by(cls.country)\
            .all()

    @classmethod
    def get_all_as_dict(cls):
        countries = {}
        for my_country in cls.get_all():
            countries[my_country.country_code] = my_country
        return countries

    @classmethod
    def get_by_id(cls, other_id):
        return db.session.query(cls)\
            .filter(cls.id == other_id)\
            .one()

    @classmethod
    def get_germany(cls):
        return db.session.query(cls)\
            .filter(cls.country_code == 'DE')\
            .one_or_none()

    @classmethod
    def find_by_country_code_and_country_and_who_region_id(cls, i_country_code, i_country, my_region):
        return db.session.query(cls).filter(
            and_(
                cls.country_code == i_country_code,
                cls.country == i_country,
                cls.region_id == my_region.id
            )
        ).one_or_none()

    @classmethod
    def find_by_country_code(cls, i_country_code):
        return db.session.query(cls).filter(
            cls.country_code == i_country_code
        ).one_or_none()

    @classmethod
    def get_by_country_code(cls, i_country_code):
        return db.session.query(cls).filter(
            cls.country_code == i_country_code
        ).one()

    @classmethod
    def find_by_country(cls, i_country):
        return db.session.query(cls).filter(
            cls.country == i_country
        ).one_or_none()

    @classmethod
    def get_by_country_code(cls, i_country_code):
        return db.session.query(cls).filter(
            cls.country_code == i_country_code
        ).one()

    @classmethod
    def get_by_country(cls, i_country):
        return db.session.query(cls).filter(
            cls.country == i_country
        ).one()

    @classmethod
    def get_who_countries_for_region(cls, region, page):
        return db.session.query(cls).filter(
            cls.region == region
        ).order_by(cls.country).paginate(page, per_page=ITEMS_PER_PAGE)


class RkiData(db.Model):
    __tablename__ = 'rki_cases'

    id = db.Column(db.Integer, primary_key=True)

    fid = db.Column(db.String(255), nullable=False)
    altersgruppe = db.Column(db.String(255), nullable=False)
    geschlecht = db.Column(db.String(255), nullable=False)
    anzahl_fall = db.Column(db.String(255), nullable=False)
    anzahl_todesfall = db.Column(db.String(255), nullable=False)
    meldedatum = db.Column(db.String(255), nullable=False)
    datenstand = db.Column(db.String(255), nullable=False)
    neuer_fall = db.Column(db.String(255), nullable=False)
    neuer_todesfall = db.Column(db.String(255), nullable=False)
    ref_datum = db.Column(db.String(255), nullable=False)
    neu_genesen = db.Column(db.String(255), nullable=False)
    anzahl_genesen = db.Column(db.String(255), nullable=False)
    ist_erkrankungsbeginn = db.Column(db.String(255), nullable=False)
    altersgruppe2 = db.Column(db.String(255), nullable=False)
    meldedatum_id = db.Column(db.Integer,
                                 db.ForeignKey('rki_cases_date_reported.id'), nullable=False)
    meldedatum = db.relationship(
        'RkiMeldedatum',
        lazy='joined',
        cascade='save-update',
        order_by='desc(RkiMeldedatum.datum)')
    landkreis_id = db.Column(db.Integer,
                           db.ForeignKey('rki_cases_landkreis.id'), nullable=False)
    landkreis = db.relationship(
        'RkiLandkreis',
        lazy='joined',
        cascade='save-update',
        order_by='asc(RkiLandkreis.landkreis)')

    @classmethod
    def remove_all(cls):
        for one in cls.get_all():
            db.session.delete(one)
        db.session.commit()
        return None

    @classmethod
    def get_all_as_page(cls, page):
        return db.session.query(cls).paginate(page, per_page=ITEMS_PER_PAGE)

    @classmethod
    def get_all(cls):
        return db.session.query(cls).all()

    @classmethod
    def get_by_id(cls, other_id):
        return db.session.query(cls).filter(cls.id == other_id).one()

    @classmethod
    def find_by_id(cls, other_id):
        return db.session.query(cls).filter(cls.id == other_id).one_or_none()
